import 'package:analyzer/dart/analysis/analysis_context.dart';
import 'package:analyzer/dart/analysis/results.dart';
import 'package:analyzer/dart/analysis/session.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/visitor.dart';
import 'package:config_widget/src/const.dart';

Map<String, dynamic> catchMap = {};
List<ConfigType> defaultList = [
  ConfigType.configDefault,
  ConfigType.noDisplay,
  ConfigType.disableTap,
];

Future<Map<String, dynamic>> analyzeSingleFile(AnalysisContext context, String path) async {
  AnalysisSession session = context.currentSession;
  ResolvedUnitResult resultA = await session.getResolvedUnit(path);

  bool isContainConfigWidget = resultA.libraryElement.imports.any((element) {
    return element.importedLibrary.toString().contains('config_widget');
  });

  if (!isContainConfigWidget) {
    return null;
  }

  CompilationUnit unit = resultA.unit; //AST 的根结点

  catchMap = {};

  for (CompilationUnitMember unitMember in unit.declarations) {
    if (unitMember is ClassDeclaration) {
      ConstructorVisit constructorVisit = ConstructorVisit(
        List.from(defaultList.map((e) => descriptionConfigType(e)).toList()),
        {},
      );
      unitMember.visitChildren(constructorVisit);
    }
  }
  return catchMap;
}

// ///测试使用的 查看当前节点定义的类型
// void recursiveVisit(var astNode) {
//   if (astNode is AstNode) {
//     if (astNode.childEntities.length == 0) {
//       print("${astNode.runtimeType} ${astNode.toString()}");
//     }
//
//     for (SyntacticEntity entity3 in astNode.childEntities) {
//       recursiveVisit(entity3);
//     }
//   }
// }

//访问的是
/* ReturnStatementImpl
 * ---InstanceCreationExpressionImpl ---
 * ------ConstructorNameImpl
 * ------ArgumentListImpl
 * ---------NamedExpressionImpl
 * ---------NamedExpressionImpl
 */

class ConstructorVisit extends RecursiveAstVisitor<void> {
  List<String> list;
  Map<String, dynamic> map;

  ConstructorVisit(this.list, this.map);
  @override
  void visitConstructorName(ConstructorName node) {
    if (node.type.name.toString() == "ConfigAdapter") {
      var parent = node.parent;
      list = List.from(defaultList.map((e) => descriptionConfigType(e)).toList());
      map = {};
      map.addAll({'itemDescription': list});
      map.addAll({'value': ConfigType.configDefault.index});

      //访问ArgumentList
      parent.childEntities.forEach((entry) {
        if (entry is ArgumentList) {
          NamedExpressionVisit namedExpressionVisit = NamedExpressionVisit(
            list,
            map,
          );
          entry.visitChildren(namedExpressionVisit);
        }
      });
      String configKey = map['configKey'];
      map.remove('configKey');
      catchMap[configKey] = map;
    }
    return;
  }
}

class NamedExpressionVisit extends RecursiveAstVisitor<void> {
  NamedExpressionVisit(this.list, this.map);
  final List<String> list;
  final Map<String, dynamic> map;
  // String configKey;
  @override
  void visitNamedExpression(NamedExpression node) {
    if (node.name.label.name == "configKey") {
      var expression = node.expression;
      // currentKey = getString(expression);
      map.addAll({'configKey': getString(expression)});
    }
    if (node.name.label.name == 'configDescription') {
      map.addAll({node.name.label.name: getString(node.expression)});
    }
    if (node.name.label.name == "itemDescription") {
      list.add(getString(node.expression));
    }
    super.visitNamedExpression(node);
  }
}

String getString(var expression) {
  if (expression is SimpleStringLiteral) {
    return expression.value;
  } else {
    return expression.toString();
  }
}
