import 'dart:io';
import 'package:pubspec_yaml/pubspec_yaml.dart';
import 'package:path/path.dart' as path;

Future<List<String>> containConfigWidgetPackagePath(String rootPath) async {
  final pubspecYaml = File('$rootPath/pubspec.yaml').readAsStringSync().toPubspecYaml();
  final List<String> packages = [];

  if (isContainConfig(pubspecYaml)) {
    packages.add(rootPath);
  }
  final List<PackageDependencySpec> blocPackage = pubspecYaml.dependencies.where((element) {
    return element.path != null;
  }).toList();

  blocPackage.forEach((element) {
    String packagePath = path.join(rootPath, element.path.path);
    final PubspecYaml pubspecYaml = File('$packagePath/pubspec.yaml').readAsStringSync().toPubspecYaml();
    bool isContain = isContainConfig(pubspecYaml);
    if (isContain) {
      packages.add(packagePath);
    }
  });
  return await searchPaths(packages);
}

bool isContainConfig(PubspecYaml pubspecYaml) {
  return pubspecYaml.dependencies.any((element) => element.package().contains('config_widget'));
}

Future<List<String>> searchPaths(List<String> packagePaths) async {
  List<String> searchPaths = packagePaths.map((element) {
    return path.join(
      element,
      "lib",
    );
  }).toList();

  List<String> resultList = [];
  for (String element in searchPaths) {
    Directory directory = Directory(element);

    await directory.list(recursive: true).forEach((entry) async {
      if (entry is File) {
        resultList.add(entry.path);
      }
    });
  }

  return resultList;
}
