import 'package:config_widget/src/config_item.dart';
import 'package:config_widget/src/config_manager.dart';
import 'package:config_widget/src/const.dart';
import 'package:flutter/material.dart';

class ConfigAdapter extends StatefulWidget {
  ///当前的key
  final String configKey;

  ///Widget 的描述信息
  final String configDescription;

  ///对于key的描述
  final List<ConfigItemWidget> children; //FieldDeclaration

  ConfigAdapter({
    @required this.configKey,
    @required this.configDescription,
    this.children,
  });

  @override
  _ConfigAdapterState createState() => _ConfigAdapterState();
}

class _ConfigAdapterState extends State<ConfigAdapter> {
  ConfigType configType;
  @override
  void initState() {
    configType = ConfigManager().getConfigType(widget.configKey);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    assert(widget.children.isNotEmpty, 'ConfigAdapter children is not empty');
    switch (configType) {
      case ConfigType.configDefault: //不处理
        return widget.children.first;
      case ConfigType.noDisplay: //不显示
        return Container();
      case ConfigType.disableTap: //不可以点击
        return IgnorePointer(
          child: Stack(
            children: <Widget>[
              widget.children.first,
            ],
          ),
        );
      case ConfigType.other: //其他的数据
      default:
        {
          int value = ConfigManager().getConfigValue(widget.configKey);
          if (widget.children != null && widget.children.length > 0 && value - 2 < widget.children.length) {
            return widget.children[value - 2];
          } else {
            return widget.children.first;
          }
        }
    }
  }
}

var test = {
  "configKey9": {
    "value": 1,
  },
  "configKey5": {
    "value": 2,
  },
  "configKey4": {
    "value": 1,
  },
  "configKey8": {
    "value": 3,
  },
  "configKey3": {
    "value": 0,
  },
  "configKey2": {
    "value": 0,
  },
  "configKey1": {
    "value": 0,
  },
  "Column1": {
    "value": 0,
  },
  "Column2": {
    "value": 0,
  },
  "configKey7": {
    "value": 0,
  },
  "configKey6": {
    "value": 0,
  }
};
